<?php

namespace Controller;

use Util\InstanceProvider;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response;
use \Service\Product as ProductService;
use Zeedhi\Framework\DTO\Request\Row as RequestRow;
use Zeedhi\Framework\DTO\Request\Filter as RequestFilter;
use Zeedhi\Framework\DTO\Request\DataSet as RequestDataset;
use Zeedhi\DevOps\Util\Environment;


class PmIntegration
{
    public function findTaskResource(RequestFilter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $resources = InstanceProvider::getService()->getPmIntegration()->findTaskResource($conditions);
            $response->addDataSet(new DataSet('taskResource', $resources));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function findTaskParent(RequestFilter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $taskParent = InstanceProvider::getService()->getPmIntegration()->findTaskParent($conditions);
            $response->addDataSet(new DataSet('taskResource', $taskParent));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function sponsorWithCrud(RequestFilter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $sponsorWithCrud = InstanceProvider::getService()->getPmIntegration()->sponsorWithCrud($conditions);
            $response->addDataSet(new DataSet('sponsorWithCrud', $sponsorWithCrud));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestDataset $request
     * @param Response $response
     */
    public function saveMultipleProjectTask(RequestDataset $request, Response $response)
    {
        try{
            $row = $request->getDataSet()->getRows()[0];
            $ids = InstanceProvider::getService()->getPmIntegration()->saveMultipleProjectTask($row);
            $response->addDataset(new DataSet("ID", $ids));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }

    }

    /**
     * @param RequestDataset $request
     * @param Response $response
     */
    public function commentsIssuesSubRelease(RequestDataset $request, Response $response)
    {
        try{
            $row = $request->getDataSet()->getRows()[0];
            $errorLog = InstanceProvider::getService()->getPmIntegration()->commentsIssuesSubRelease($row);
            $response->addDataset(new DataSet('errorLog', $errorLog));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }

    }

    public function commentsIssuesRelease(RequestDataset $request, Response $response)
    {
        try{
            $row = $request->getDataSet()->getRows()[0];
            $errorLog = InstanceProvider::getService()->getPmIntegration()->commentsIssuesRelease($row);
            $response->addDataset(new DataSet('errorLog', $errorLog));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }

    }

    public function saveNewProjectTask(RequestDataset $request, Response $response)
    {
        try{
            $row = $request->getDataSet()->getRows()[0];
            $errorLog = InstanceProvider::getService()->getPmIntegration()->saveNewProjectTask($row);
            $response->addDataset(new DataSet('ID', $errorLog));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }

    }
}